#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
@author: Martin Schoenball

version history:
2018/06/13: first version
2018/11/11: v2: changed channel naming to be in line with miniSEED Guide
2020/02/04: Added an if statement to assign station names based on the time period to reflect 
            replacement of dead channels on 2020/02/04
"""
import os
import obspy

def read(fname):
    stations = ('PDB01', 'PDB02', 'PDB03', 'PDB04', 'PDB05', 'PDB06', 'PDB07', 'PDB08', 'PDB09', 'PDB10', 'PDB11', 'PDB12', 
                'OT01', 'OT02', 'OT03', 'OT04', 'OT05', 'OT06', 'OT07', 'OT08', 'OT09', 'OT10', 'OT11', 'OT12', 
                'PDT1', 'PDT1', 'PDT1', 'PDB3', 'PDB3', 'PDB3', 'PDB4', 'PDB4', 'PDB4', 'PDB6', 'PDB6', 'PDB6', 
                'PSB7', 'PSB7', 'PSB7', 'PSB9', 'PSB9', 'PSB9', 'PST10', 'PST10', 'PST10', 'PST12', 'PST12', 'PST12', 
                'OB13', 'OB13', 'OB13', 'OB15', 'OB15', 'OB15', 'OT16', 'OT16', 'OT16', 'OT18', 'OT18', 'OT18', 'CMon', 'CTrig', 'CEnc', 'PPS')
    stations2020 = ('PDB01', 'PDB02', 'PDB03', 'PDB04', 'PDB05', 'PDB06', 'PDB07', 'PDB08', 'PDB09', 'PDB10', 'PDB11', 'PDB12', 
                'OT01', 'OT02', 'OT03', 'OT04', 'OT05', 'OT06', 'OT07', 'OT08', 'OT09', 'OT10', 'OT11', 'OT12', 
                'PDT2', 'PDT2', 'PDT2', 'PDB3', 'PDB3', 'PDB3', 'PDB4', 'PDB4', 'PDB4', 'PDB6', 'PDB6', 'PDB6', 
                'PSB7', 'PSB7', 'PSB7', 'PSB9', 'PSB9', 'PSB9', 'PST10', 'PST10', 'PST10', 'PST12', 'PST12', 'PST12', 
                'OB13', 'OB13', 'OB13', 'OB15', 'OB15', 'OB15', 'OT17', 'OT17', 'OT17', 'OT18', 'OT18', 'OT18', 'CMon', 'CTrig', 'CEnc', 'PPS')
    location = ('', '', '', '', '', '', '', '', '', '', '', '', 
                '', '', '', '', '', '', '', '', '', '', '', '', 
                '', '', '', '', '', '', '', '', '', '', '', '', 
                '', '', '', '', '', '', '', '', '', '', '', '', 
                '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '')
    channel = ('XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 
               'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1', 'XN1',  
               'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 
               'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 
               'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', 'XNZ', 'XNX', 'XNY', '', '', '', '')
    
    HEADER_SIZE=4
    HEADER_OFFSET=27
    DATA_OFFSET=148
    VOLTAGE_RANGE=10
    clock_channel = 63
    f = open(fname, "rb")

    f.seek(HEADER_OFFSET, os.SEEK_SET)
    # read header
    H = np.fromfile(f, dtype=np.int32, count=HEADER_SIZE)
    BUFFER_SIZE=H[0]
    FREQUENCY=H[1]
    NUM_OF_BUFFERS=H[2]
    channels=H[3]
    # read data
    f.seek(DATA_OFFSET, os.SEEK_SET)
    A = np.fromfile(f, dtype=np.int32, count = BUFFER_SIZE*NUM_OF_BUFFERS)
    A = 2*VOLTAGE_RANGE*np.reshape(A, (len(A)/channels, channels)) - VOLTAGE_RANGE
    A = A/4294967296.0

    # arrange it in an obspy stream
    tr = []
    for s in range(channels):
        tr.append(obspy.Trace(data=A[:,s]))
    st = obspy.Stream(tr)
    path, fname = os.path.split(fname)

    try:
        time_to_first_full_second = np.where(A[:,clock_channel]>(2e7/2**31))[0][0]-3
        if time_to_first_full_second > 101000:
            print('Cannot read time signal')
        # in case we start during the time pulse
        if time_to_first_full_second < 0:
            time_to_first_full_second = np.where(A[50000:,clock_channel]>(2e7/2**31))[0][0]-3
            
        starttime = obspy.UTCDateTime(np.int(fname[5:9]), np.int(fname[9:11]), np.int(fname[11:13]), np.int(fname[13:15]), 
                                      np.int(fname[15:17]), np.int(fname[17:19]), np.int(1e6*(1 - (np.float(time_to_first_full_second)/FREQUENCY))))
    except Exception as e:
        print('Cannot read time exact signal: ' + fname + '. Taking an approximate one instead')
        starttime = obspy.UTCDateTime(np.int(fname[5:9]), np.int(fname[9:11]), np.int(fname[11:13]), np.int(fname[13:15]), 
                                      np.int(fname[15:17]), np.int(fname[17:19]), np.int(1e2*np.int(fname[19:23])) )

    for ii in range(channels):
        st[ii].stats.network = 'SV'
        if starttime < obspy.UTCDateTime(2020,2,4,21,10,0):
            st[ii].stats.station = stations[ii]
        else:
            st[ii].stats.station = stations2020[ii]
        st[ii].stats.location = location[ii]
        st[ii].stats.channel = channel[ii]
        st[ii].stats.starttime = starttime
        st[ii].stats.sampling_rate = H[1]
    return st

